from .partial import PartialChapter, PartialGroup
from .relation import Relation
types = {0: None, 1: "Shounen", 2: "Seinen", 3: "Josei", 4: "Shoujo"}
statuses = {0: None, 1: "ongoing", 2: "completed"}


class Manga:
    """Represents a MangaDex Manga."""
    __slots__ = ("id", "title", "titles", "desc", "artist", "author", "language", "status", "type", "tag_ids",
                 "last_chapter", "last_volume", "hentai", "links", "relations", "ratings", "views", "follows",
                 "comments", "last_upload", "cover", "chapters", "groups", "session")

    def __init__(self, data, session, chaps=None, groups=None):
        self.id = data["id"]
        self.title = data["title"]
        self.titles = data["altTitles"]
        self.desc = data["description"]
        self.artist = data["artist"]
        self.author = data["author"]
        self.language = data["publication"]["language"]
        self.status = statuses[data["publication"]["status"]]
        self.type = types[data["publication"]["demographic"]]
        self.tag_ids = data["tags"]
        self.last_chapter = data["lastChapter"]
        self.last_volume = data["lastVolume"]
        self.hentai = data["isHentai"]
        self.links = data["links"]
        self.relations = [Relation(x) for x in data["relations"] or []]
        self.ratings = data["rating"]
        self.views = data["views"]
        self.follows = data["follows"]
        self.comments = data["comments"]
        self.last_upload = data["lastUploaded"]
        self.cover = data["mainCover"]
        self.chapters = [PartialChapter(x) for x in chaps or []]
        self.groups = [PartialGroup(x) for x in groups or []]
        self.session = session

    def get_tags(self):
        tags = self.session.get("https://api.mangadex.org/v2/tag").json()["data"]
        return [tags[x] for x in tags if int(x) in self.tag_ids]

    def get_covers(self):
        return self.session.get(f"https://api.mangadex.org/v2/manga/{self.id}/covers").json()["data"]



from .partial import PartialGroup


class Chapter:
    """Represents a MangaDex Chapter."""
    __slots__ = ("id", "hash", "manga_id", "manga_title", "volume", "chapter", "title", "language", "groups",
                 "uploader", "timestamp", "thread_id", "views", "pages", "server", "fallback_server", "server_status",
                 "session")

    def __init__(self, data, session):
        self.id = data["id"]
        self.hash = data["hash"]
        self.manga_id = data["mangaId"]
        self.manga_title = data["mangaTitle"]
        self.volume = data["volume"]
        self.chapter = data["chapter"]
        self.title = data["title"]
        self.language = data["language"]
        self.groups = [PartialGroup(x) for x in data["groups"]]
        self.uploader = data["uploader"]
        self.timestamp = data["timestamp"]
        self.thread_id = data["threadId"]
        self.views = data["views"]
        self.pages = data["pages"]
        self.server = data["server"]
        self.fallback_server = data["serverFallback"]
        self.server_status = data["status"]
        self.session = session

    def get_page(self, page, fallback=False):
        if fallback:
            return self.fallback_server + self.hash + "/" + self.pages[page]
        else:
            return self.server + self.hash + "/" + self.pages[page]

    def format_url(self, img, fallback=False):
        if fallback:
            return self.fallback_server + self.hash + "/" + img
        else:
            return self.server + self.hash + "/" + img

    def refresh(self):
        p = {"saver": "data-saver" in self.server, "mark_read": False}
        req = self.session.get(f"https://api.mangadex.org/v2/chapter/{self.id}", params=p)
        if req.status_code == 200:
            data = req.json()["data"]
            self.pages = data["pages"]
            self.server = data["server"]
            self.fallback_server = data["serverFallback"]



import requests
import time
from .manga import Manga
from .chapter import Chapter
from .group import Group
from .user import User, UserSettings, UserFollow, UserUpdate


class APIError(Exception):
    pass


class LoginError(Exception):
    pass


class MangaDex:
    """Represents the MangaDex API Client."""
    def __init__(self):
        self.url = "https://mangadex.org"
        self.api = "https://api.mangadex.org/v2"
        self.session = requests.Session()
        self.login_success = False

    def login(self, username: str, password: str):
        """Logs in to MangaDex using an username and a password."""
        url = f"{self.url}/ajax/actions.ajax.php?function=login"
        credentials = {"login_username": username, "login_password": password}
        headers = {
            "method": "POST",
            "path": "/ajax/actions.ajax.php?function=login",
            "scheme": "https",
            "Accept": "*/*",
            "accept-encoding": "gzip, deflate, br",
            "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
            "origin": self.url,
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "x-requested-with": "XMLHttpRequest",
        }

        post = self.session.post(url, data=credentials, headers=headers)
        if not post.cookies.get("mangadex_session"):
            print("Invalid MangaDex credentials. Some API calls will be unavailable.")
            raise LoginError("Invalid credentials.")
        else:
            self.login_success = True
            return True

    def logout(self):
        """Resets the current session."""
        self.session = requests.Session()
        self.login_success = False

    def get_manga(self, id_: int, full=False) -> Manga:
        """Gets a manga with a specific id. Set full to True to populate Manga.chapters and Manga.groups"""
        p = None
        if full:
            p = {"include": "chapters"}
        req = self.session.get(f"{self.api}/manga/{id_}", params=p)

        if req.status_code == 200:
            json = req.json()["data"]
            if full:
                return Manga(json["manga"], self.session, json["chapters"], json["groups"])
            else:
                return Manga(json, self.session)
        else:
            self.raise_err(req)

    def get_chapter(self, id_: int, low_quality=False, mark_read=False) -> Chapter:
        """Gets a chapter with a specific id."""
        p = {"saver": low_quality, "mark_read": mark_read}
        req = self.session.get(f"{self.api}/chapter/{id_}", params=p)

        if req.status_code == 200:
            return Chapter(req.json()["data"], self.session)
        else:
            self.raise_err(req)

    def get_group(self, id_: int, full=False) -> Group:
        """Gets a group with a specific id. Set full to True to populate Group.chapters_uploaded and Group.collabs."""
        p = None
        if full:
            p = {"include": "chapters"}
        req = self.session.get(f"{self.api}/group/{id_}", params=p)

        if req.status_code == 200:
            json = req.json()["data"]
            if full:
                return Group(json["group"], json["chapters"], json["groups"])
            else:
                return Group(json)
        else:
            self.raise_err(req)

    def get_user(self, id_: int = 0, full=False) -> User:
        """Gets an user with a specific id. Set full to True to populate User.chapters_uploaded and User.groups."""
        p = None
        if full:
            p = {"include": "chapters"}
        if id_ == 0 and self.login_success:
            id_ = "me"
        req = self.session.get(f"{self.api}/user/{id_}", params=p)

        if req.status_code == 200:
            json = req.json()["data"]
            if full:
                return User(json["user"], json["chapters"], json["groups"])
            else:
                return User(json)
        else:
            self.raise_err(req)

    def get_user_settings(self, id_: int = 0) -> UserSettings:
        """Gets an user's settings. To retrieve another user's settings than the one currently logged in, you must
        be a MangaDex staff member."""
        if id_ == 0 and self.login_success:
            id_ = "me"
        req = self.session.get(f"{self.api}/user/{id_}/settings")

        if req.status_code == 200:
            json = req.json()["data"]
            return UserSettings(json)
        else:
            self.raise_err(req)

    def get_user_list(self, id_: int = 0, follow_type: int = 0, hentai_mode: int = 1) -> []:
        """Gets an user's manga list. This settings follows the privacy mode of user's MDList."""
        p = {"hentai": hentai_mode}
        if follow_type != 0:
            p["type"] = follow_type
        if id_ == 0 and self.login_success:
            id_ = "me"
        req = self.session.get(f"{self.api}/user/{id_}/followed-manga", params=p)

        if req.status_code == 200:
            json = req.json()["data"]
            if json:
                return [UserFollow(x) for x in json]
        else:
            self.raise_err(req)

    def get_user_updates(self, id_: int = 0, follow_type: int = 0, hentai_mode: int = 1, delayed=False,
                         include_blocked=False) -> []:
        """Gets an user's manga feed. To retrieve another user's feed than the one currently logged in, you must be a
        MangaDex staff member."""
        p = {"type": follow_type, "hentai": hentai_mode, "delayed": delayed, "blockgroups": include_blocked}
        if id_ == 0 and self.login_success:
            id_ = "me"
        req = self.session.get(f"{self.api}/user/{id_}/followed-updates", params=p)

        if req.status_code == 200:
            json = req.json()["data"]["chapters"]
            if json:
                return [UserUpdate(x) for x in json]
        else:
            self.raise_err(req)

    def get_user_ratings(self, id_: int = 0) -> {}:
        """Gets an user's ratings."""
        if id_ == 0 and self.login_success:
            id_ = "me"
        req = self.session.get(f"{self.api}/user/{id_}/ratings")

        if req.status_code == 200:
            json = req.json()["data"]
            if json:
                return {x["mangaId"]: x["rating"] for x in json}
        else:
            self.raise_err(req)

    def get_user_manga(self, id_: int, uid: int = 0) -> UserFollow:
        """Gets an user's manga from their MDList."""
        if uid == 0 and self.login_success:
            uid = "me"
        req = self.session.get(f"{self.api}/user/{uid}/manga/{id_}")

        if req.status_code == 200:
            json = req.json()["data"]
            return UserFollow(json)
        else:
            self.raise_err(req)

    def set_user_markers(self, mangas: list, read: bool, id_: int = 0):
        """Sets chapters as read or unread."""
        reqs = []
        lists = [mangas[x:x + 100] for x in range(0, len(mangas), 100)]
        if id_ == 0 and self.login_success:
            id_ = "me"

        for y in lists:
            p = {"chapters": y, "read": read}
            reqs.append(self.session.post(f"{self.api}/user/{id_}/marker", json=p))
            time.sleep(1)

        if reqs[-1].status_code == 200:
            return True
        else:
            self.raise_err(reqs[-1])

    def raise_err(self, req):
        print("There was an error while handling this API Request. Please make sure that you're logged in"
              " if you're trying to access personal data. Refer to the docs for more information.")
        raise APIError(f"API Request Error. Status={req.status_code} LoggedIn={self.login_success}")



from .chapter import Chapter
import time
from pathlib import Path


def dl_page(chapter: Chapter, page, path, fallback=False):
    """Downloads a single page."""
    with chapter.session.get(chapter.format_url(page, fallback)) as p:
        with open(str(Path(path + "/" + page)), "wb") as f:
            f.write(p.content)


def dl_chapter(chapter: Chapter, path, fallback=False, time_controller: int = 1):
    """Downloads an entire chapter."""
    chapter.refresh()
    y = 0
    for x in chapter.pages:
        dl_page(chapter, x, path, fallback)
        if time_controller:
            time.sleep(time_controller)
        y += 1



from .partial import PartialChapter, PartialGroup


class Group:
    """Represents a MangaDex Group."""
    __slots__ = ("id", "name", "names", "language", "leader", "members", "desc", "website", "discord", "irc_server",
                 "irc_channel", "email", "founded_on", "follows", "views", "chapters", "thread_id", "thread_posts",
                 "locked", "inactive", "delay", "last_update", "banner", "chapters_uploaded", "collabs")

    def __init__(self, data, chaps=None, groups=None):
        self.id = data["id"]
        self.name = data["name"]
        self.names = data["altNames"]
        self.language = data["language"]
        self.leader = data["leader"]
        self.members = data["members"]
        self.desc = data["description"]
        self.website = data["website"]
        self.discord = data["discord"]
        self.irc_server = data["ircServer"]
        self.irc_channel = data["ircChannel"]
        self.email = data["email"]
        self.founded_on = data["founded"]
        self.follows = data["follows"]
        self.views = data["views"]
        self.chapters = data["chapters"]
        self.thread_id = data["threadId"]
        self.thread_posts = data["threadPosts"]
        self.locked = data["isLocked"]
        self.inactive = data["isInactive"]
        self.delay = data["delay"]
        self.last_update = data["lastUpdated"]
        self.banner = data["banner"]
        self.chapters_uploaded = [PartialChapter(x) for x in chaps or []]
        self.collabs = [PartialGroup(x) for x in groups or []]



from setuptools import setup

with open("README.md", "r") as f:
    long_description = f.read()

setup(name='MangaDex.py',
      packages=[
          "MangaDexPy"
      ],
      version='1.0.1',
      description='An API wrapper for MangaDex.',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/Proxymiity/MangaDex.py',
      download_url='https://github.com/Proxymiity/MangaDex.py/releases',
      author='Proxymiity',
      author_email='commits@proxymiity.fr',
      license='MIT',
      install_requires=[
          'requests>=2.25.0',
      ],
      zip_safe=False
      )
